%%
% LPVIQCExample.m
% Example with parameter-varying IQC


%% Define delayed systems, Fu(G,Stau*I)
DelayDim = 2;

% System 1 (rho = 0)
a = [-2.7 -0.5; 0 -2];
b = [0.5 0.3 0; 0 0.3 -0.3];
c = [0.5 0; -2 1; 1 0];
G1 = ss(a,b,c,zeros(3));

% System 2 (rho = 1)
a = [-3.5 0.5; 1 -3];
b = [0.2 -0.6 0; -1.6 0.5 -0.5];
c = [-3 0.5; 0.7 0.5; -0.4 0];
G2 = ss(a,b,c,zeros(3));


%% "Exact" LTI gains for large delay
% This is approximately computed from gridded frequency response.
tau = 100;
Dtau = ss(1);
Dtau.InputDelay = tau;
Stau = Dtau-1;

w = logspace(-2,3,1e3);

sys1fr = frd( lft(Stau*eye(2),G1), w);
g1 = norm( sys1fr , 'inf' );
fprintf('\n');
fprintf('Exact gain for tau = %4.3f and LTI system 1  = %4.3f\n',tau,g1);

sys2fr = frd( lft(Stau*eye(2),G2), w);
g2 = norm( sys2fr , 'inf' );
fprintf('Exact gain for tau = %4.3f and LTI system 2  = %4.3f\n',tau,g2);
fprintf('\n');


%% LTI IQC Analysis for two systems
[giqclmi1,info1] = simpledelaygain(G1,DelayDim);
fprintf('IQC gain bound for LTI system 1 = %4.3f\n',giqclmi1);

[giqclmi2,info2] = simpledelaygain(G2,DelayDim);
fprintf('IQC gain bound for LTI system 2 = %4.3f\n',giqclmi2);
fprintf('\n');


%% LPV IQC Analysis
Glpv = cat(3,G1,G2);  

% constant IQC
[giqclmi3,info3] = simpledelaygain(Glpv,DelayDim);
fprintf('IQC gain bound for LPV system with constant Q = %4.3f\n',giqclmi3);

% parameter varying IQC
constantQ=false;
[giqclmi4,info4] = simpledelaygain(Glpv,DelayDim,constantQ);
fprintf('IQC gain bound for LPV system with parameter-varying Q = %4.3f\n',giqclmi4);
fprintf('\n');


%% Plot dependence of Q vs. parameter value
rho = [0,1]; %scheduling parameter
N = 2; %number of grid points

Q = reshape(info4.Q,[4 2]);
figure(1);
subplot(221)
plot(rho,Q(1,:),0,info1.Q(1,1),'ro',1,info2.Q(1,1),'ro'); hold on;
plot(rho,repmat(info3.Q(1,1),[N 1]),'g'); hold off;
ylabel('Q11');

subplot(222)
plot(rho,Q(3,:),0,info1.Q(1,2),'ro',1,info2.Q(1,2),'ro'); hold on;
plot(rho,repmat(info3.Q(1,2),[N 1]),'g'); hold off;
ylabel('Q12');

subplot(223)
plot(rho,Q(2,:),0,info1.Q(2,1),'ro',1,info2.Q(2,1),'ro'); hold on;
plot(rho,repmat(info3.Q(2,1),[N 1]),'g'); hold off;
ylabel('Q21');
xlabel('rho');

subplot(224)
plot(rho,Q(4,:),0,info1.Q(2,2),'ro',1,info2.Q(2,2),'ro'); hold on;
plot(rho,repmat(info3.Q(2,2),[N 1]),'g'); hold off;
ylabel('Q22');
xlabel('rho');

